unit TestButton;

{ This toy application demonstrates an apparent flaw in the generation
  of STAB debug data by FPC 2.3.1 SVN 8427. When compiled with -g and
  run under GDB 6.6 with a breakpoint placed where commented below,
  GDB barfs when one attempts to step into TChild.Create.

  Using objdump 2.17 to look at the STABS (objdump -g) reports:
  
    Bad stab: 145;-2147483647;2147483647;
    
  This flaw is very sensitive. If you make any of the changes to the
  source (commented below), the problem goes away. Fortunately this
  has afforded me a work around (swapping the order of the TParent
  and TChild class forward declarations, which is possible in the
  large application from which this toy project was derived).
  
  Note that the flaw reported by objdump seems to apply to any object
  (I have) created with FPC 2.3.1 SVN 8427 not just objects that trip
  up GDB 6.6

  I hope this helps track this one down. bruce@causal.com }

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, StdCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end; 

// swap the order of these declarations (TParent & TChild) and the problem is fixed.
  TParent = class;
  TChild = class;

  TParent = class(TComponent)
  private
    FChild : TChild; // remove me and the problem is fixed.
  public
    constructor Create ( AOwner : TComponent ); override;
  end;

  TChild = class(TParent)
  private
    FField : Integer; // remove me and the problem is fixed.
  public
    constructor Create ( AOwner : TComponent ); override;
  end;

var
  Form1: TForm1; 

implementation

{ TForm1 }

procedure TForm1.Button1Click(Sender: TObject);
begin
  TChild.Create(nil); // break-point here and try to step into constructor (gdb/stabs)
end;

{ TParent }

constructor TParent.Create(AOwner: TComponent);
begin
  Inherited Create(AOwner);
end;

{ TChild }

constructor TChild.Create(AOwner: TComponent);
begin
  Inherited;
end;

initialization
  {$I testbutton.lrs}

end.

